# -*- coding: utf-8 -*-
"""Shared test cases."""

from tests import test_lib as shared_test_lib


class DecrypterTestCase(shared_test_lib.BaseTestCase):
  """The unit test case for decrypter object implementations."""
